* Examples used in %JM Macro Reference Manual;
* Alberto Garcia-Hernandez, Dimitris Rizopoulow;

libname  mydata  '/sas/user/u100683/JM/Data';                              *Change data location accordingly.;
                                                                           *You may need to replace / by \ depending on your environment;
options mautolocdisplay mautosource 
        sasautos = ("/sas/user/u100683/JM/macros/" sasautos);        *Change %JM macro location accordingly;
                                                                           *You may need to replace / by \ depending on your environment;

options  mprint nodate center;
%let folder = /sas/user/u100683/JM/Output/;                         * Change for the desired output location accordingly;
                                                                    * Do NOT quote the folder;
                                                                    * You may need to replace / by \ depending on your environment;
%JM(Data = mydata.aids,
    SubjectVar = patient,
	LongiType  = normal,
    LongiTimeModel = linear, 
    LongiVar = cd4,
    LongiTimevar = obstime, 
    LongiTimeInter = ddi, 
    LongiGMatrix = un,
    EventTimeVar = time, 
    EventVar = death, 
    EventVal = 1,
    EventModel = piecewise, 
    EventCovariates = ddi,
	InitialParameters = disjoint,
    NLMIXEDOptions	= qpoints=3,	
    OutputParameters = Example_params,
    SharedParam	= current_value,
	AdditionalOptions = calculateexectime);


* The macro call above has calculated for us the knots of the Event model;
* We need to obtain them because we are going to call EXACTLY THE SAME MODEL on a new dataset with the subjects WE want dynamic predictions for;
* For the new run WE need to provide the knots manually;
* This step would not be neccesary if WE would have set the knots manually above;
* This step would be neccessary for the longituidinal model too should we have used knots in the longituidinal model;

proc sql noprint;
   select min(time), max(time) into :lowb, :uppb
   from  mydata.aids where death=1;
quit; 

proc sql noprint;
select * into :kn1,:kn2,:kn3,:kn4,:kn5,:kn6 from _JMCALCULATED_KNOTS_EVENT_MODEL;
quit;
 
proc transpose data=_JMCALCULATED_KNOTS_EVENT_MODEL out=knots (rename=(col1=TimeTo) drop=_name_ _label_);
run;


* Let's verify that we obtain the same fit with a call specifying ther knots ourselves;
* This step is unecessary, it is just a check;

%JM(Data = mydata.aids,
    SubjectVar = patient,
	LongiType  = normal,
    LongiTimeModel = linear, 
    LongiVar = cd4,
    LongiTimevar = obstime, 
    LongiTimeInter = ddi, 
    LongiGMatrix = un,
    EventTimeVar = time, 
    EventVar = death, 
    EventVal = 1,
    EventModel = piecewise, 
    Eventknot1 = &kn1., Eventknot2 = &kn2., Eventknot3 = &kn3., 
    Eventknot4 = &kn4., Eventknot5 = &kn5., Eventknot6 = &kn6., 
	EventLowerknot 	= &lowb.,	
	EventUpperknot 	= &uppb.,
    EventCovariates = ddi,
	InitialParameters = disjoint,
    NLMIXEDOptions	= qpoints=3,	
    OutputParameters = Example_params,
    SharedParam	= current_value,
	AdditionalOptions = calculateexectime);

*  This is the dataset of the subjects we want dynapic predictions for;
*  This could be a) the entire collection of censored values in the original dataset;
*  b) a subset of these, or c) some other data points not used for fitting the model;
*  In our case we are going to obtain dynamic predictions for two made up subjects not used to fit the model above;

data subjects_to_predict;
patient="Subject 1";
ddi=1;
time=6.1;death=0;   * We know the subject was alive at time 5;
   * Longitudinal responses for this subject;
   obstime=0;
   cd4=5;
   output;
   obstime=6;
   cd4=7;
   output;
patient="Subject 2";
ddi=0;
time=12.5;death=0;  * We know the subject was alive at time 10;
   * Longitudinal responses for this subject;
   obstime=0;
   cd4=3;
   output;
   obstime=6;
   cd4=6;
   output;
   obstime=12;
   cd4=9;
   output;
patient="Subject 3";
ddi=0;
time=12.5;death=0;  * We know the subject was alive at time 10;
   * Longitudinal responses for this subject;
   obstime=0;
   cd4=7;
   output;
   obstime=6;
   cd4=5;
   output;
   obstime=12;
   cd4=2;
   output;
run;


* Now we run the same model on the "subjects_to_predict" dataset;
* Should we want predictions for the subjects used to fit the model we would not need two runs to the macro but just one saving the predicted bi0 and bi1;
* We will get a warning because we do not have enough data to estimate covariance matrix;

%JM(Data = subjects_to_predict,                                      /*IMPORTANT CHANGE, NOW WE POINT AT THE NEW DATA SET*/
    SubjectVar = patient,
	LongiType  = normal,
    LongiTimeModel = linear, 
    LongiVar = cd4,
    LongiTimevar = obstime, 
    LongiTimeInter = ddi, 
    LongiGMatrix = un,
    EventTimeVar = time, 
    EventVar = death, 
    EventVal = 1,
    EventModel = piecewise, 
    Eventknot1 = &kn1., Eventknot2 = &kn2., Eventknot3 = &kn3.,       /*IMPORTANT CHANGE NOW WE FIX MANUALLY THE KNOTS*/
    Eventknot4 = &kn4., Eventknot5 = &kn5., Eventknot6 = &kn6., 
	EventLowerknot 	= &lowb.,	                                      /*IMPORTANT CHANGE NOW WE FIX MANUALLY THE KNOTS*/
	EventUpperknot 	= &uppb.,                                         /*IMPORTANT CHANGE NOW WE FIX MANUALLY THE KNOTS*/
    EventCovariates = ddi,
	InitialParameters = Example_params,                               /*IMPORTANT CHANGE WE USE THE PARAMETERS ESTIMATED ABOVE*/
    NLMIXEDOptions	= qpoints=3 tech=none,                /*IMPORTANT CHANGE WE DO NOT WANT TO OPTIMIZE BUT KEEP THE INPUT PARAMETERS FIXED*/	
    SharedParam	= current_value,
	OutputPredictions 	= bi0 bi1,                                    /*IMPORTANT CHANGE NOW WE SAVE THE RANDOM EFFECTS*/
	OutputPredictData 	= bi0 bi1,                                    /*IMPORTANT CHANGE NOW WE SAVE THE RANDOM EFFECTS*/
	AdditionalOptions = calculateexectime);


* Arranging the obtained empirical bayes estimates of the random effects for the subjects we want to obtain dynamic predictions;

proc sort data=bi0 (keep= patient pred ddi death time rename=(pred=bi0)) out=bi0_ nodupkey;
by patient;
run;

proc sort data=bi1 (keep= patient pred ddi death time rename=(pred=bi1)) out=bi1_ nodupkey;
by patient;
run;

* Creating a dataset with b) empirical bayes estimates of the random effects and b) the collection of time points u to evaluate pi(u);
* See Rizopopulus (2011) for the equation  pi(u)=Surv(u)/Surv(t) where t is the time we know the subject was still alive;

data timepoints;
merge bi0_ bi1_;
by patient;
cd4=.;
obstime=.;
*Assume we are only interestid until max observed time;
do predtime = time to &uppb. by 0.01;
patient_time=patient!!" on time "!!left(put(predtime,z8.3));
output;
end;
run;

* In this call to the macro we will calculte the cumulative hazard at the given timepoints and given the random effects;
* It is key to use Random_Effects_in_Input_data=YES so we do not fit a random effects model but take the saved bi0 and bi as constants;
* It is also important to use SubjectVar = patient_time so JM thinks they are all times from independent individuals;
* We use OutputPredictions 	= cum_hi and OutputPredictData 	= cum_hi to store the cumultive hazards;
* We will get a warning because we do not have enough data to estimate covariance matrix;

%JM(Data = timepoints,                                               /*IMPORTANT CHANGE, NOW WE POINT AT THE NEW DATA SET*/
    SubjectVar = patient_time,                                       /*IMPORTANT CHANGE NOW WE USE patient_time to identify subjects*/
	LongiType  = normal,
    LongiTimeModel = linear, 
    LongiVar = cd4,
    LongiTimevar = obstime, 
    LongiTimeInter = ddi, 
    LongiGMatrix = un,
    EventTimeVar = predtime, 
    EventVar = death, 
    EventVal = 1,
    EventModel = piecewise, 
    Eventknot1 = &kn1., Eventknot2 = &kn2., Eventknot3 = &kn3.,       /*IMPORTANT CHANGE NOW WE FIX MANUALLY THE KNOTS*/
    Eventknot4 = &kn4., Eventknot5 = &kn5., Eventknot6 = &kn6., 
	EventLowerknot 	= &lowb.,	                                      /*IMPORTANT CHANGE NOW WE FIX MANUALLY THE KNOTS*/
	EventUpperknot 	= &uppb.,                                         /*IMPORTANT CHANGE NOW WE FIX MANUALLY THE KNOTS*/
    EventCovariates = ddi,
	InitialParameters = Example_params,                               /*IMPORTANT CHANGE WE USE THE PARAMETERS ESTIMATED ABOVE*/
    NLMIXEDOptions	= /*gconv=0*/ qpoints=1 tech=none,                /*IMPORTANT CHANGE WE DO NOT WANT TO OPTIMIZE BUT KEEP THE INPUT PARAMETERS FIXED*/	
    SharedParam	= current_value,
	OutputPredictions 	= cum_hi,                                     /*IMPORTANT CHANGE NOW WE SAVE THE CUMULATIVE HAZARD AT THE EVALUATED POINTS*/
	OutputPredictData 	= cum_hi,                                     /*IMPORTANT CHANGE NOW WE SAVE THE CUMULATIVE HAZARD AT THE EVALUATED POINTS*/ 
	AdditionalOptions = calculateexectime,
    Random_Effects_in_Input_data=YES);                                /*IMPORTANT CHANGE WE WANT TO USE THE EMPIRICAL BAYES ESTIMATES OF THE RANDOM EFFECTS AS IF THEY WERE CONSTANTS */


* Arranging the obtained cumulative hazards in order to obtain pi(u) ;
* See Rizopopulus (2011) for the equation  pi(u)=Surv(u)/Surv(t) where t is the time we know the subject was still alive;

proc sort data=cum_hi (keep= patient pred ddi death time predtime rename=(pred=cum_hi)) out=cum_hi_ ;
by patient predtime;
run;

data dynamic_predictions;
  set cum_hi_;
  by patient predtime;
  retain surv_t;
  if first.patient then surv_t=.;
  if predtime=time then surv_t=exp(-cum_hi);
  surv_u=exp(-cum_hi);
  pi_u=surv_u/surv_t;
run;

 * Plotting the dynamic predictions;

proc sgplot data=dynamic_predictions;
  xaxis label="Time (months)" min=&lowb. max=&uppb.;
  yaxis label="Predicted survival given acumulated information" values=(0 to 1 by 0.2);
  series x=predtime y=pi_u /group=patient;
run;


* We observe that the predicted survival times over time have some undesired fluctuations due to the fact that we approximate ;
* the cumulative hazard using the using the 15-points Gauss-Kronrod rule over a non-continuous function;
* One way to improve this is not using the standard data structure by a vertical data structure in the last call to the macro;
* Doing so the macro will estimate for us the cumulative hazard using the 15-points Gauss-Kronrod rule for each the intervals defined by;
* the knots of the piecewise exponential;








* In the below code I create a new dataset timepoints_vertical simuilar to timepoints;
* but creating for  each of the "patient_time"'s one row per piecewise exponetial interval ;
* We also create the JMDist variable required for DataStructure=VERTICAL;

data knot2;
if _n_=1 then TimeFrom=0; 
set knots end=last;
output;
TimeFrom=TimeTo;
retain TimeFrom;
if last then do;
TimeTo=&uppb+1;
output;
end;
run;


proc sql noprint;
create table timepoints_vertical
as select *,min(TimeTo, predtime) as predtime2, "EVENT" as JMDist
   from knot2,timepoints 
   where  TimeFrom<PredTime 
   order by patient_time, TimeFrom, predtime ;
quit;


* In this call to the macro we will calculte the cumulative hazard at the given timepoints and given the random effects;
* Now for each patient_time and each predtime we will estimate a cumulative hazard for each the intervals defined by;
* the knots of the piecewise exponential.;
 

 
%JM(Data = timepoints_vertical,                                      /*IMPORTANT CHANGE, NOW WE POINT AT THE NEW DATA SET*/
    DataStructure=VERTICAL,                                           /*IMPORTANT CHANGE, DATA STRUCTURE IS VERTICAL*/
    SubjectVar = patient_time,                                       
	LongiType  = normal,
    LongiTimeModel = linear, 
    LongiVar = cd4,
    LongiTimevar = obstime, 
    LongiTimeInter = ddi, 
    LongiGMatrix = un,
	EventTimeVar = predtime2,                                        /*IMPORTANT CHANGE, NOW WE CALL PREDTIME2*/
    EventTimeStart = TimeFrom,                                       /*IMPORTANT CHANGE, NOW WE USE TIMEFROM*/
    EventVar = death, 
    EventVal = 1,
    EventModel = piecewise, 
    Eventknot1 = &kn1., Eventknot2 = &kn2., Eventknot3 = &kn3.,       
    Eventknot4 = &kn4., Eventknot5 = &kn5., Eventknot6 = &kn6., 
	EventLowerknot 	= &lowb.,	                                      /*IMPORTANT CHANGE NOW WE FIX MANUALLY THE KNOTS*/
	EventUpperknot 	= &uppb.,                                         /*IMPORTANT CHANGE NOW WE FIX MANUALLY THE KNOTS*/
    EventCovariates = ddi,
	InitialParameters = Example_params,                               /*IMPORTANT CHANGE WE USE THE PARAMETERS ESTIMATED ABOVE*/
    NLMIXEDOptions	= /*gconv=0*/ qpoints=1 tech=none,                /*IMPORTANT CHANGE WE DO NOT WANT TO OPTIMIZE BUT KEEP THE INPUT PARAMETERS FIXED*/	
    SharedParam	= current_value,
	OutputPredictions 	= cum_hi,                                     /*IMPORTANT CHANGE NOW WE SAVE THE CUMULATIVE HAZARD AT THE EVALUATED POINTS*/
	OutputPredictData 	= cum_hi_plus,                                /*IMPORTANT CHANGE NOW WE SAVE THE CUMULATIVE HAZARD AT THE EVALUATED POINTS*/ 
	AdditionalOptions = calculateexectime,
    Random_Effects_in_Input_data=YES);                                /*IMPORTANT CHANGE WE WANT TO USE THE EMPIRICAL BAYES ESTIMATES OF THE RANDOM EFFECTS AS IF THEY WERE CONSTANTS */


* Arranging the obtained cumulative hazards in order to obtain pi(u) ;

proc sort data=cum_hi_plus (keep= patient patient_time pred ddi death time TimeFrom predtime predtime2 rename=(pred=cum_hi)) out=cum_hi_plus_ ;
by patient predtime TimeFrom predtime2;
run;

* Now for each patient_time and each predtime we sum all cumulative hazard for each the sum intervals defined by;
* the knots of the piecewise exponential so obtain the cum hazard until predtime.;

data cum_hi_plus_2;
  set cum_hi_plus_;
  by patient predtime;
  retain total_cum_hi;
  if first.predtime then total_cum_hi=0;
  total_cum_hi=total_cum_hi+cum_hi;
  if last.predtime then output;
  drop predtime2 cum_hi;
run;

data dynamic_predictions_plus;
  set cum_hi_plus_2;
  by patient predtime;
  retain surv_t;
  if first.patient then surv_t=.;
  if predtime=time then surv_t=exp(-total_cum_hi);
  surv_u=exp(-total_cum_hi);
  pi_u=surv_u/surv_t;
run;

proc sgplot data=dynamic_predictions_plus;
  xaxis label="Time (months)" min=&lowb. max=&uppb.;
  yaxis label="Predicted survival given acumulated information" values=(0 to 1 by 0.2);
  series x=predtime y=pi_u /group=patient;
run;
